use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
BSON
Binary
Bool
Bytes
Code
DBRef
Daniels
David
Decimal128
Doc
Double
Duclos
Eric
Golden
Gunn
Inc
Int32
Int64
Kostyuk
MaxKey
MinKey
MongoDB
OID
OIDs
ObjectID
ObjectId
Oleg
Olivier
PCRE
PP
Pat
Petr
Písař
Raw
Regex
Riedel
Stefan
String
Time
Timestamp
Types
VERSIONING
Yury
Zavarin
and
bson
codec
codecs
cub
david
dbrefs
eric
ixhash
lib
minimalist
numification
odc
oid
pgunn
ppisar
sharding
tock
yury
